<?php
/**
* Authentification class
*
* The Authentification class checks if the username/password combination matches 
* the database user table.
*
* @copyright	CONTREXX CMS - Astalavista IT Engineering GmbH Thun
* @author		Astalavista Development Team <thun@astalvista.ch>
* @access		public
* @module		authentification
* @modulegroup	core
* @version		1.0.0
*/  


class Auth
{
    var $type = ''; // values are frontend or backend     	
    var $errorMessage = array();
    var $status;
	
	/**
	 * Constructor
	 *
	 */	
    function Auth($authType='frontend')
    {
		$this->type=$authType;				
	}
	
	function checkAuth()
	{
		global $sessionObj, $_CORELANG;	
		
		$this->assignData();
			
		if (isset($_SESSION['auth']['username']) && isset($_SESSION['auth']['password'])) { 		
			///////////////////////////////////
			// Backend Authentification
			///////////////////////////////////	
			if ($this->type=='backend') {								
				if (!$this->checkCode()) {
				    $this->errorMessage[] = $_CORELANG['TXT_SECURITY_CODE_IS_INCORRECT'];
				    return false;
				}
		    	if ($this->checkUserData() && $this->checkType()) {	
		    		// sets cookie for 30 days						
					setcookie("username", $_SESSION['auth']['username'], time()+3600*24*30);
					$this->log();															
					$sessionObj->cmsSessionUserUpdate($username=$_SESSION['auth']['username']);																			
					return true;
				}
			}else {
				///////////////////////////////////
				// Frontend Authentification
				///////////////////////////////////
		    	if($this->checkUserData() && $this->checkType()) {																						
					$sessionObj->cmsSessionUserUpdate($username=$_SESSION['auth']['username']);																			
					return true;
				}
			}
		}
		$sessionObj->cmsSessionUserUpdate($username="unknown");
		$sessionObj->cmsSessionStatusUpdate($status=$this->type);
		return false;
	}
	
    /**
     * Assign data from login form to internal values
     *
     * This function takes the values for username and password
     * from $_POST and assigns them to internal variables.
     *
     * @access private
     * @global $_POST
     * @see    Auth
     * @return void
     */
    function assignData()
    {
        if (isset($_POST['USERNAME']) && $_POST['USERNAME'] != '') {
            $_SESSION['auth']['username'] = (get_magic_quotes_gpc() == 1 ? stripslashes($_POST['USERNAME']) : $_POST['USERNAME']);
        }

        if (isset($_POST['PASSWORD']) && $_POST['PASSWORD'] != '') {
        	$_SESSION['auth']['password'] = md5($_POST['PASSWORD']);
        }
        
        if (isset($_POST['secid2']) && $_POST['secid2'] != '') {
            $_SESSION['auth']['secid2'] = (get_magic_quotes_gpc() == 1 ? stripslashes($_POST['secid2']) : $_POST['secid2']);
        }
    }
    
    /**
     * checks the code from the security image
     *
     * This function compares the security image code with the
     * Code in the session     
     *
     * @access private
     * @global $_SESSION
     * @see    Auth
     * @return void
     */
    function checkCode()
    {
		if ($_SESSION['auth']['secid2'] != $_SESSION['auth']['secid']) {
		    return false;
		} else {
		    return true;
		}
    }    	
	
    /**
     * check user data from the database
     *
     * check user data from the database.
     *
     * @access private
     * @global $objDatabase
     * @see    Auth
     * @return void
     */
    function checkUserData()
    {
		global $objDatabase, $_CORELANG;
		
		$arrGroups = array();
		$arrUserGroups = array();
		
		if (isset($_SESSION['auth']['username']) && isset($_SESSION['auth']['password'])) {
			$user = $objDatabase->qstr($_SESSION['auth']['username']);
			
			$query = "SELECT password,
			                       username,
			                       id,
			                       groups,
			                       is_admin,
			                       langId,
			                       firstname,
			                       lastname,
			                       active
			                  FROM ".DBPREFIX."access_users 
			                 WHERE username = ".$user." AND active=1";
			
			$objResult = $objDatabase->SelectLimit($query, 1);
			
			if ($objResult !== false) {
				if ($_SESSION['auth']['password'] == $objResult->fields['password']) {
					$_SESSION['auth']['userid'] = $objResult->fields['id'];
					$_SESSION['auth']['username'] = $objResult->fields['username'];
					$_SESSION['auth']['name'] = $objResult->fields['firstname']." ".$objResult->fields['lastname'];
					$_SESSION['auth']['lang'] = $objResult->fields['langId'];
					$_SESSION['auth']['is_admin'] = $objResult->fields['is_admin'];
					
					$arrGroups = explode(",", $objResult->fields['groups']);
					
					$objResult = $objDatabase->Execute("SELECT group_id FROM ".DBPREFIX."access_user_groups WHERE is_active=1");
					if ($objResult !== false) {
						while (!$objResult->EOF) {
							if (in_array($objResult->fields['group_id'], $arrGroups)) {
								array_push($arrUserGroups, $objResult->fields['group_id']);
							}
							$objResult->MoveNext();
						}
					}
					$_SESSION['auth']['groups'] = $arrUserGroups;
					
					$this->_initUserRights();
					return true;
				} else {
					$this->errorMessage[] = $_CORELANG['TXT_PASSWORD_OR_USERNAME_IS_INCORRECT'];
					return false;
				}
			} else {
				$this->errorMessage[] = $_CORELANG['TXT_PASSWORD_OR_USERNAME_IS_INCORRECT'];
				return false;
			}
	    }
	    return false;
    }    
	
	function _initUserRights()
	{
		global $objDatabase;
		
		$sqlWhereString = "";
		$arrStaticRightIds = array();
		$arrDynamicRightIds = array();
		
		$arrUserGroups = $_SESSION['auth']['groups'];
		foreach ($arrUserGroups as $groupId) {
			$sqlWhereString .= "group_id=".intval($groupId)." OR ";
		}
		$sqlWhereString = substr($sqlWhereString, 0, strlen($sqlWhereString)-4);
		
		if (count($arrUserGroups)>0) {
			// get static right ids
			$objResult = $objDatabase->Execute("SELECT access_id FROM ".DBPREFIX."access_group_static_ids WHERE ".$sqlWhereString);
			
			if ($objResult !== false) {
				while (!$objResult->EOF) {
					array_push($arrStaticRightIds, $objResult->fields['access_id']);
					$objResult->MoveNext();
				}
			}
			
			// get dynamic right ids
			$objResult = $objDatabase->Execute("SELECT access_id FROM ".DBPREFIX."access_group_dynamic_ids WHERE ".$sqlWhereString);
			
			if ($objResult !== false) {
				while (!$objResult->EOF) {
					array_push($arrDynamicRightIds, $objResult->fields['access_id']);
					$objResult->MoveNext();
				}
			}
			
			$_SESSION['auth']['static_access_ids'] = array_unique($arrStaticRightIds);
			$_SESSION['auth']['dynamic_access_ids'] = array_unique($arrDynamicRightIds);
			
			sort($_SESSION['auth']['static_access_ids']);
			sort($_SESSION['auth']['dynamic_access_ids']);
		}
	}
    
	function checkType()
	{
	    global $objDatabase;
	    
    	if ($_SESSION['auth']['is_admin']==1)
    	    return true;
    	$arrUserGroups = $_SESSION['auth']['groups'];
    	if (is_array($arrUserGroups)) {
    		$objResult = $objDatabase->Execute("SELECT group_id FROM ".DBPREFIX."access_user_groups WHERE type='$this->type'");
    		if ($objResult !== false) {
    			while (!$objResult->EOF) {
					if (in_array($objResult->fields['group_id'],$arrUserGroups))
    			    	return true;
    				$objResult->MoveNext();
    			}
    		}
    	}
	    return false;
	}
    
	function log()
	{
	    global $objDatabase;
	    
		if (!isset($_SESSION['auth']['log'])) {
			$remote_host = @gethostbyaddr($_SERVER['REMOTE_ADDR']);		
			$referer = get_magic_quotes_gpc() ? strip_tags((strtolower($_SERVER['HTTP_REFERER']))) : addslashes(strip_tags((strtolower($_SERVER['HTTP_REFERER']))));
			$httpUserAgent = get_magic_quotes_gpc() ? strip_tags($_SERVER['HTTP_USER_AGENT']) : addslashes(strip_tags($_SERVER['HTTP_USER_AGENT']));
			$httpAcceptLanguage = get_magic_quotes_gpc() ? strip_tags($_SERVER['HTTP_ACCEPT_LANGUAGE']) : addslashes(strip_tags($_SERVER['HTTP_ACCEPT_LANGUAGE']));
            	
			$objDatabase->Execute("INSERT INTO ".DBPREFIX."log 
					                    SET userid=".intval($_SESSION['auth']['userid']).",
					                        datetime = ".$objDatabase->DBTimeStamp(time()).",
				                            useragent = '".$httpUserAgent."',
				                            userlanguage = '".$httpAcceptLanguage."',
				                            remote_addr = '".strip_tags($_SERVER['REMOTE_ADDR'])."',
				                            remote_host = '".$remote_host."',
				                            http_x_forwarded_for = '".strip_tags($_SERVER['HTTP_X_FORWARDED_FOR'])."',
				                            http_via = '".strip_tags($_SERVER['HTTP_VIA'])."',
				                            http_client_ip = '".strip_tags($_SERVER['HTTP_CLIENT_IP'])."',
											referer ='".$referer."'");
			$_SESSION['auth']['log']=true;
		}
	}
	
	/**
	* Function logout
	*  
	* kills all sessions and redirects to the frontend page   	                                                                       
	*/	
	function logout()
	{	
		if (isset($_SESSION['auth'])) {
			unset($_SESSION['auth']);
			session_destroy();
		}
		
		if ($this->type=='public') {
		    header('Location: ?section=login');
		} else {
		    header('Location: ../index.php'); 
		}
		exit;
	}
	
	/**
	* Function status
	*
	* @global    array      $_SESSION[]                                                                         
	*/
	function status()
	{
		global $sessionObj, $_CORELANG;
		
		if (!isset($_SESSION['auth']['username'])) {
			return "";
		} else {
			return $_CORELANG['TXT_LOGGED_IN_AS']." ".$_SESSION['auth']['username']." (<a href='?section=logout' title='logout'>logout</a>)";
		}
	}
	
	/**
	* Lost password
	*
	* Sends an email with instructions on how to reset a password of an user account
	*
	* @access public
	* @global object $objDatabase
	* @global object $objTemplate
	* @global array $_CORELANG
	* @global array $_CONFIG
	*/
	function lostPassword(&$objTemplate)
	{
		global $objDatabase, $_CORELANG, $_CONFIG;
		
		// set language variables
		$objTemplate->setVariable(array(
			'TXT_LOST_PASSWORD_TEXT'	=> $_CORELANG['TXT_LOST_PASSWORD_TEXT'],
			'TXT_EMAIL'					=> $_CORELANG['TXT_EMAIL'],
			'TXT_RESET_PASSWORD'		=> $_CORELANG['TXT_RESET_PASSWORD']
		));
		
		if (isset($_POST['restore_pw'])) {
			$email = contrexx_addslashes($_POST['email']);
			
			// check if the user account exists
			$objResult = $objDatabase->Execute("SELECT username, password FROM ".DBPREFIX."access_users WHERE active=1 AND email='".$email."'");
			if ($objResult !== false) {
				if ($objResult->RecordCount() == 1) {
					$restoreKey = md5($objResult->fields['username'].$objResult->fields['password'].time());
					
					if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users SET restore_key='".$restoreKey."', restore_key_time='".(time()+3600)."' WHERE email='".$email."'") !== false) {
						$sendto = $email;
						$subject = $_SERVER['SERVER_NAME'].": ".$_CORELANG['TXT_RESET_PASSWORD'];
						
						if ($this->type == "frontend") {
							$restorLink = strtolower(ASCMS_PROTOCOL)."://".$_SERVER['SERVER_NAME'].ASCMS_PATH_OFFSET."/index.php?section=login&cmd=resetpw&username=".$objResult->fields['username']."&restoreKey=".$restoreKey;
						} else {
							$restorLink = strtolower(ASCMS_PROTOCOL)."://".$_SERVER['SERVER_NAME'].ASCMS_PATH_OFFSET."/admin/index.php?cmd=resetpw&username=".$objResult->fields['username']."&restoreKey=".$restoreKey;
						}
						
						$message = str_replace(array("%USERNAME%", "%URL%", "%SENDER%"), array($objResult->fields['username'], $restorLink, $_CONFIG['coreAdminName']), $_CORELANG['TXT_RESTORE_PASSWORD_MAIL']);
						$headers  = "MIME-Version: 1.0";
						$headers .= "Content-type: text/html; charset=iso-8859-1";
						$headers .= "To: " .$sendto."\r\n" ;
						$headers .= "From: ".$sendto."\r\n";
						
						if (@mail($sendto, $subject, $message, $headers)) {
							$statusMessage = str_replace("%EMAIL%", $email, $_CORELANG['TXT_LOST_PASSWORD_MAIL_SENT']);
							if ($objTemplate->blockExists('login_lost_password')) {
								$objTemplate->hideBlock('login_lost_password');
							}
						} else {
							$statusMessage = str_replace("%EMAIL%", $email, $_CORELANG['TXT_EMAIL_NOT_SENT']);
						}
					} else {
						$statusMessage = $_CORELANG['TXT_DATABASE_QUERY_ERROR'];
					}
				} else {
					$statusMessage = $_CORELANG['TXT_ACCOUNT_WITH_EMAIL_DOES_NOT_EXIST']."<br />";
				}
			} else {
				$statusMessage = $_CORELANG['TXT_DATABASE_QUERY_ERROR'];
			}
			
			$objTemplate->setVariable(array(
				'LOGIN_STATUS_MESSAGE'		=> $statusMessage
			));
		}
	}
	
	/**
	* Reset password
	*
	* Reset the password of an user by using an reset formular
	*
	* @access public
	* @global object $objDatabase
	* @global object $objTemplate
	* @global array $_CORELANG
	*/
	function resetPassword(&$objTemplate)
	{
		global $objDatabase, $_CORELANG;
		
		$_POST['username'] = isset($_POST['username']) ? contrexx_strip_tags($_POST['username']) : (isset($_GET['username']) ? contrexx_strip_tags($_GET['username']) : "");
		$_POST['restore_key'] = isset($_POST['restore_key']) ? contrexx_strip_tags($_POST['restore_key']) : (isset($_GET['restoreKey']) ? contrexx_strip_tags($_GET['restoreKey']) : "");
		$_POST['password'] = isset($_POST['password']) ? contrexx_strip_tags($_POST['password']) : "";
		$_POST['password2'] = isset($_POST['password2']) ? contrexx_strip_tags($_POST['password2']) : "";
		
		$objResult = $objDatabase->Execute("SELECT id FROM ".DBPREFIX."access_users WHERE username='".$_POST['username']."' AND restore_key!='' AND restore_key='".$_POST['restore_key']."' AND restore_key_time>='".time()."' AND active=1");
		if ($objResult !== false) {
			if ($objResult->RecordCount() == 1) {
				if (isset($_POST['reset_password'])) {
					if (strlen($_POST['password'])<6) {
						$statusMessage .= $_CORELANG['TXT_INVALID_PASSWORD']."<br />";
					} elseif ($_POST['username'] == $_POST['password']) {
						$statusMessage .= $_CORELANG['TXT_PASSWORD_NOT_USERNAME_TEXT']."<br />";
					} elseif ($_POST['password'] != $_POST['password2']) {
						$statusMessage .= $_CORELANG['TXT_PW_DONOT_MATCH']."<br />";
					} else {
						$password = md5($_POST['password']);
						if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users SET password='".$password."', restore_key='', restore_key_time='' WHERE username='".$_POST['username']."' AND restore_key='".$_POST['restore_key']."' AND active=1") !== false) {
							$statusMessage = $_CORELANG['TXT_PASSWORD_CHANGED_SUCCESSFULLY'];
							if ($objTemplate->blockExists('login_reset_password')) {
								$objTemplate->hideBlock('login_reset_password');
							}
						} else {
							$statusMessage = $_CORELANG['TXT_DATEBASE_QUERY_ERROR'];
						}
					}
				}
			} else {
				if ($objTemplate->blockExists('login_reset_password')) {
					$objTemplate->hideBlock('login_reset_password');
				}
				
				$statusMessage = $_CORELANG['TXT_INVALID_USER_ACCOUNT'];
			}
			
			$objTemplate->setVariable(array(
				'TXT_USERNAME'				=> $_CORELANG['TXT_USERNAME'],
				'TXT_PASSWORD'				=> $_CORELANG['TXT_PASSWORD'],
				'TXT_VERIFY_PASSWORD'		=> $_CORELANG['TXT_VERIFY_PASSWORD'],
				'TXT_PASSWORD_MINIMAL_CHARACTERS'	=> $_CORELANG['TXT_PASSWORD_MINIMAL_CHARACTERS'],
				'TXT_SET_PASSWORD_TEXT'		=> $_CORELANG['TXT_SET_PASSWORD_TEXT'],
				'TXT_SET_NEW_PASSWORD'		=> $_CORELANG['TXT_SET_NEW_PASSWORD'],
			));
			
		} else {
			$statusMessage = $_CORELANG['TXT_DATEBASE_QUERY_ERROR'];
		}
		$objTemplate->setVariable(array(
			'LOGIN_STATUS_MESSAGE'	=> $statusMessage,
			'LOGIN_USERNAME'		=> $_POST['username'],
			'LOGIN_RESTORE_KEY'		=> $_POST['restore_key']
		));
	}
	
	/**
    * Returns a textual error message
    *  
    * @access public
    * @return string   error message
    */
    function errorMessage()
    {
        $_errorMessage ='';
        foreach ($this->errorMessage as $value) {
            $_errorMessage .= $value."<br />";
        }
        return $_errorMessage;
    }
}
?>
