<?php
/**
* Class community
*
* class with methodes to manage the community
*
* @copyright CONTREXX CMS - Astalavista IT Engineering GmbH Thun
* @author Astalavista Development Team <thun@astalvista.ch>
* @module community
* @modulegroups core_modules
* @access public
* @version 1.0.0
*/

require_once ASCMS_MODULE_PATH.'/community/lib/communityLib.class.php';

class Community extends Community_Library
{
	/**
	* Template object
	*
	* @access private
	* @var object
	*/
	var $_objTpl;
	
	/**
	* Status message
	*
	* @access private
	* @var string
	*/
	var $_statusMessage = "";
	
	/**
	* Constructor
	*
	* @param string $pageContent
	* @see __construct()
	*/
	function Community($pageContent)
	{
		$this->__construct($pageContent);
	}
	
	/**
	* PHP5 constructor
	*
	* @param string $pageContent
	* @see HTML_Template_Sigma(), HTML_Template_Sigma::setErrorHandling(), HTML_Template_Sigma::setTemplate(), initialize()
	*/
	function __construct($pageContent)
	{
		$this->_objTpl = &new HTML_Template_Sigma('.');
		$this->_objTpl->setErrorHandling(PEAR_ERROR_DIE);
		$this->_objTpl->setTemplate($pageContent);
		
		$this->initialize();
	}
	
	/**
	* Get community page
	*
	* @access public
	* @see _register(), _showRegisterPage(), _activate(), _showActivationPage(), _profile(), _showProfilePage(), $_objTpl, HTML_Template_Sigma::get()
	* @return string content
	*/
	function getCommunityPage()
	{
		if (!isset($_GET['cmd'])) {
			$_GET['cmd'] = "";
		}
		
		switch ($_GET['cmd']) {
		case 'register':
			$this->_register();
			$this->_showRegisterPage();
			break;
			
		case 'activate':
			$this->_activate();
			$this->_showActivationPage();
			break;
			
		case 'profile':
			$this->_profile();
			$this->_showProfilePage();
			break;
		
		default:
			break;
		}
		
		return $this->_objTpl->get();
	}
	
	/**
	* Register a new user
	*
	* @access private
	* @global array $_ARRAYLANG
	* @global integer $_LANGID
	*/
	function _register()
	{
		global $_ARRAYLANG, $_LANGID;
		
		$status = true;
		
		if (isset($_POST['register'])) {
			$objValidator = &new FWValidator();
			$objUser = &new FWUser();
			
			$_POST['username'] = contrexx_strip_tags($_POST['username']);
			$_POST['email'] = contrexx_strip_tags($_POST['email']);
			$_POST['password'] = contrexx_strip_tags($_POST['password']);
			$_POST['password2'] = contrexx_strip_tags($_POST['password2']);
			
			if (empty($_POST['username']) || empty($_POST['email']) || empty($_POST['password']) || empty($_POST['password2'])) {
				$this->_statusMessage .= $_ARRAYLANG['TXT_FILL_OUT_ALL_REQUIRED_FIELDS']."<br />";
				$status = false;
			} else {
				if (!$objUser->checkUsernameIntegrity($_POST['username'])) {
					$this->_statusMessage .= $_ARRAYLANG['TXT_USERNAME_ALREADY_USED']."<br />";
					$status = false;
				} else {
					if (!$objValidator->isEmail($_POST['email'])) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_INVALID_EMAIL_ADDRESS']."<br />";
						$status = false;
					} else {
						if (!$objUser->checkEmailIntegrity($_POST['email'])) {
							$this->_statusMessage .= $_ARRAYLANG['TXT_EMAIL_ALREADY_USED']."<br />";
							$status = false;
						}
					}
					if (strlen($_POST['password'])<6) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_INVALID_PASSWORD']."<br />";
						$status = false;
					} elseif ($_POST['username'] == $_POST['password']) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_PASSWORD_NOT_USERNAME_TEXT']."<br />";
						$status = false;
					} elseif ($_POST['password'] != $_POST['password2']) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_PW_DONOT_MATCH']."<br />";
						$status = false;
					}
				}
			}
			
			if ($status) {
				$arrGroups = explode(',', $this->arrConfig['community_groups']['value']);
				if ($this->arrConfig['user_activation']['status']) {
					$activationKey = md5($_POST['username'].$_POST['password'].time());
					
					if ($objUser->addUser($_POST['username'], 0, $_POST['password'], $_POST['email'], "", "", $_LANGID, $arrGroups, 0, $activationKey, time() + ($this->arrConfig['user_activation_timeout']['value'] * 3600)) !== false) {
						$sendto = $_POST['email'];
						$subject = str_replace("%HOST%", $_SERVER['SERVER_NAME'], $_ARRAYLANG['TXT_CONFIRM_REGISTRATION']);
						$activationLink = "http://".$_SERVER['SERVER_NAME']."/index.php?section=community&cmd=activate&username=".$_POST['username']."&activationKey=".$activationKey;
						$hostLink = "http://".$_SERVER['SERVER_NAME'];
						$message = str_replace(array("%HOST%","%USERNAME%","%PASSWORD%", "%ACTIVATION_LINK%", "%HOST_LINK%"), array($_SERVER['SERVER_NAME'], $_POST['username'], $_POST['password'], $activationLink, $hostLink), $_ARRAYLANG['TXT_CONFIRM_REGISTRATION_MAIL']);
						$headers  = "MIME-Version: 1.0";
						$headers .= "Content-type: text/html; charset=iso-8859-1";
						$headers .= "To: " . $sendto . "<br>\n" ;
						$headers .= "From: " . $_SERVER['SERVER_NAME'];
						
						if (@mail($sendto, $subject, $message, $headers)) {
							$timeoutStr = "";
							if ($this->arrConfig['user_activation_timeout']['status']) {
								if ($this->arrConfig['user_activation_timeout']['value'] > 1) {
									$timeoutStr = $this->arrConfig['user_activation_timeout']['value']." ".$_ARRAYLANG['TXT_HOURS_IN_STR'];
								} else {
									$timeoutStr = " ".$_ARRAYLANG['TXT_HOUR_IN_STR'];
								}
								
								$timeoutStr = str_replace("%TIMEOUT%", $timeoutStr, $_ARRAYLANG['TXT_ACTIVATION_TIMEOUT']);
							}
							$this->_statusMessage = $_ARRAYLANG['TXT_USER_ACCOUNT_SUCCESSFULLY_CREATED']."<br /><br />".str_replace("%TIMEOUT%", $timeoutStr, $_ARRAYLANG['TXT_ACTIVATION_BY_USER_MSG']);
						} else {
							$mailSubject = str_replace("%HOST%", "http://".$_SERVER['SERVER_NAME'], $_ARRAYLANG['TXT_COULD_NOT_SEND_ACTIVATION_MAIL']);
							$adminEmail = '<a href="mailto:'.$_CONFIG['coreAdminEmail'].'?subject='.$mailSubject.'" title="'.$_CONFIG['coreAdminEmail'].'">'.$_CONFIG['coreAdminEmail'].'</a>';
							$this->_statusMessage = str_replace("%EMAIL%", $adminEmail, $_ARRAYLANG['TXT_COULD_NOT_SEND_EMAIL']);
						}
					} else {
						$this->_statusMessage .= $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
					}
				} else {
					if ($objUser->addUser($_POST['username'], 0, $_POST['password'], $_POST['email'], "", "", $_LANGID, $arrGroups, 0, $activationKey) !== false) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_USER_ACCOUNT_SUCCESSFULLY_CREATED']."<br /><br />";
						$this->_statusMessage .= str_replace("%HOST%", $_SERVER['SERVER_NAME'], $_ARRAYLANG['TXT_ACTIVATION_BY_SYSTEM']);
					} else {
						$this->_statusMessage .= $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
					}
				}
				
				if ($this->_objTpl->blockExists('community_registration_form')) {
					$this->_objTpl->hideBlock('community_registration_form');
				}
			} else {
				if ($this->_objTpl->blockExists('community_registration_form')) {
					$this->_objTpl->touchBlock('community_registration_form');
				}
			}
		}
	}
	
	/**
	* Activate user account
	*
	* @access private
	* @global object $objDatabase
	* @global array $_ARRAYLANG
	* @global array $_CONFIG
	*/
	function _activate()
	{
		global $objDatabase, $_ARRAYLANG, $_CONFIG;
		
		if (isset($_GET['username']) && $_GET['activationKey']) {
			$username = contrexx_addslashes($_GET['username']);
			$activationKey = contrexx_addslashes($_GET['activationKey']);
			$mailSubject = str_replace("%HOST%", "http://".$_SERVER['SERVER_NAME'], $_ARRAYLANG['TXT_ACCOUNT_ACTIVATION_NOT_POSSIBLE']);
			$adminEmail = '<a href="mailto:'.$_CONFIG['coreAdminEmail'].'?subject='.$mailSubject.'" title="'.$_CONFIG['coreAdminEmail'].'">'.$_CONFIG['coreAdminEmail'].'</a>';
			$status = true;
			
			if ($this->arrConfig['user_activation_timeout']['status']) {
				$objResult = $objDatabase->Execute("SELECT restore_key_time FROM ".DBPREFIX."access_users WHERE username='".$username."' AND restore_key='".$activationKey."'");
				if ($objResult !== false) {
					if ($objResult->RecordCount() == 1) {
						if ($objResult->fields['restore_key_time'] < time()) {
							$this->_statusMessage = $_ARRAYLANG['TXT_ACTIVATION_TIME_EXPIRED'].'<br /><a href="index.php?section=community&amp;cmd=register" title="'.$_ARRAYLANG['TXT_REGISTER_NEW_ACCOUNT'].'">'.$_ARRAYLANG['TXT_REGISTER_NEW_ACCOUNT'].'</a>';
							$status = false;
						}
					} else {
						$this->_statusMessage = str_replace("%EMAIL%", $adminEmail, $_ARRAYLANG['TXT_INVALID_USERNAME_OR_ACTIVATION_KEY']);
						$status = false;
					}
				} else {
					$this->_statusMessage = $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
				}
			}
			
			if ($status) {
				if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users SET active=1, restore_key='', restore_key_time='' WHERE username='".$username."' AND restore_key='".$activationKey."'") !== false) {
					if ($objDatabase->Affected_Rows() == 1) {
						$this->_statusMessage = $_ARRAYLANG['TXT_ACCOUNT_SUCCESSFULLY_ACTIVATED'];
					} else {
						$this->_statusMessage = str_replace("%EMAIL%", $adminEmail, $_ARRAYLANG['TXT_INVALID_USERNAME_OR_ACTIVATION_KEY']);
					}
				} else {
					$this->_statusMessage = $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
				}
			}
		} else {
			$this->_statusMessage = str_replace("%EMAIL%", $adminEmail, $_ARRAYLANG['TXT_INVALID_USERNAME_OR_ACTIVATION_KEY']);
		}
	}
	
	/**
	* Change user profile data
	*
	* @access private
	* @global object $objAuth
	* @global object $objDatabase
	* @global array $_ARRAYLANG
	*/
	function _profile()
	{
		global $objAuth, $_ARRAYLANG, $objDatabase;
		
		if ($objAuth->checkAuth()) {
			if (isset($_POST['change_profile'])) {
				$_POST['firstname'] = contrexx_strip_tags($_POST['firstname']);
				$_POST['lastname'] = contrexx_strip_tags($_POST['lastname']);
				$_POST['residence'] = contrexx_strip_tags($_POST['residence']);
				$_POST['profession'] = contrexx_strip_tags($_POST['profession']);
				$_POST['interests'] = contrexx_strip_tags($_POST['interests']);
				$_POST['webpage'] = contrexx_strip_tags($_POST['webpage']);
				
				if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users
					SET `firstname`='".$_POST['firstname']."',
						`lastname`='".$_POST['lastname']."',
						`residence`='".$_POST['residence']."',
						`profession`='".$_POST['profession']."',
						`interests`='".$_POST['interests']."',
						`webpage`='".$_POST['webpage']."'
					WHERE username='".contrexx_strip_tags($_SESSION['auth']['username'])."'") !== false) {
					$this->_statusMessage = $_ARRAYLANG['TXT_PROFILE_DATA_SUCCESSFULLY_CHANGED'];
				} else {
					$this->_statusMessage = $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
				}
				$this->_objTpl->setVariable('COMMUNITY_STATUS_MESSAGE_PROFILE', $this->_statusMessage);
			} elseif (isset($_POST['change_email'])) {
				$objUser = &new FWUser();
				$objValidator = &new FWValidator();
				
				$_POST['email'] = contrexx_strip_tags($_POST['email']);
				$_POST['email2'] = contrexx_strip_tags($_POST['email2']);
				
				if (empty($_POST['email']) || empty($_POST['email2'])) {
					$this->_statusMessage = $_ARRAYLANG['TXT_FILL_OUT_ALL_REQUIRED_FIELDS'];
				} elseif ($_POST['email'] != $_POST['email2']) {
					$this->_statusMessage = $_ARRAYLANG['TXT_EMAIL_DO_NOT_MATCH'];
				} elseif (!$objValidator->isEmail($_POST['email'])) {
					$this->_statusMessage = $_ARRAYLANG['TXT_INVALID_EMAIL_ADDRESS'];
				} elseif (!$objUser->checkEmailIntegrity($_POST['email'])) {
					$this->_statusMessage = $_ARRAYLANG['TXT_EMAIL_ALREADY_USED'];
				} else {
					if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users SET email='".$_POST['email']."' WHERE username='".contrexx_strip_tags($_SESSION['auth']['username'])."'") !== false) {
						$this->_statusMessage = $_ARRAYLANG['TXT_EMAIL_SUCCESSFULLY_CHANGED'];
					} else {
						$this->_statusMessage = $_ARRAYLANG['TXT_DATABASE_QUERY_ERROR'];
					}
				}
				
				$this->_objTpl->setVariable('COMMUNITY_STATUS_MESSAGE_EMAIL', $this->_statusMessage);
			} elseif (isset($_POST['change_password'])) {
				$_POST['password'] = contrexx_strip_tags($_POST['password']);
				$_POST['password2'] = contrexx_strip_tags($_POST['password2']);
				
				if (strlen($_POST['password'])<6) {
						$this->_statusMessage .= $_ARRAYLANG['TXT_INVALID_PASSWORD']."<br />";
				} elseif (contrexx_strip_tags($_SESSION['auth']['username']) == $_POST['password']) {
					$this->_statusMessage .= $_ARRAYLANG['TXT_PASSWORD_NOT_USERNAME_TEXT']."<br />";
				} elseif ($_POST['password'] != $_POST['password2']) {
					$this->_statusMessage .= $_ARRAYLANG['TXT_PW_DONOT_MATCH']."<br />";
				} else {
					$password = md5($_POST['password']);
					if ($objDatabase->Execute("UPDATE ".DBPREFIX."access_users SET password='".$password."', restore_key='' WHERE username='".contrexx_strip_tags($_SESSION['auth']['username'])."'") !== false) {
						$this->_statusMessage = $_ARRAYLANG['TXT_PASSWORD_CHANGED_SUCCESSFULLY'];
						$_POST['PASSWORD'] = $_POST['password'];
						$objAuth->checkAuth();
					} else {
						$this->_statusMessage = $_ARRAYLANG['TXT_DATEBASE_QUERY_ERROR'];
					}
				}
				$this->_objTpl->setVariable('COMMUNITY_STATUS_MESSAGE_PASSWORD', $this->_statusMessage);
			}
			
			
			
		} else {
			header('Location: index.php?section=login&redirect=aHR0cDovL2NvbnRyZXh4Lml0c2ljaGVyaGVpdC5jaC9pbmRleC5waHA/c2VjdGlvbj1jb21tdW5pdHkmY21kPXByb2ZpbGU=');
			exit;
		}
	}
	
	/**
	* Show the activation page
	*
	* @access private
	*/
	function _showActivationPage()
	{
		$this->_objTpl->setVariable('COMMUNITY_STATUS_MESSAGE', $this->_statusMessage);
	}
	
	/**
	* Show the registration page
	*
	* @access private
	* @global array $_ARRAYLANG
	*/
	function _showRegisterPage()
	{
		global $_ARRAYLANG;
		
		$username = "";
		$email = "";
		
		if (isset($_POST['register'])) {
			$username = $_POST['username'];
			$email = $_POST['email'];
		}
		
		$this->_objTpl->setVariable(array(
			'TXT_LOGIN_NAME'					=> $_ARRAYLANG['TXT_LOGIN_NAME'],
			'TXT_LOGIN_PASSWORD'				=> $_ARRAYLANG['TXT_LOGIN_PASSWORD'],
			'TXT_PASSWORD_MINIMAL_CHARACTERS'	=> $_ARRAYLANG['TXT_PASSWORD_MINIMAL_CHARACTERS'],
			'TXT_VERIFY_PASSWORD'				=> $_ARRAYLANG['TXT_VERIFY_PASSWORD'],
			'TXT_FIRST_NAME'					=> $_ARRAYLANG['TXT_FIRST_NAME'],
			'TXT_LAST_NAME'						=> $_ARRAYLANG['TXT_LAST_NAME'],
			'TXT_EMAIL'							=> $_ARRAYLANG['TXT_EMAIL'],
			'TXT_REGISTER'						=> $_ARRAYLANG['TXT_REGISTER'],
			'TXT_ALL_FIELDS_REQUIRED'			=> $_ARRAYLANG['TXT_ALL_FIELDS_REQUIRED'],
			'TXT_PASSWORD_NOT_USERNAME_TEXT'	=> $_ARRAYLANG['TXT_PASSWORD_NOT_USERNAME_TEXT'],
			'COMMUNITY_USERNAME'				=> $username,
			'COMMUNITY_EMAIL'					=> $email,
			'COMMUNITY_STATUS_MESSAGE'			=> $this->_statusMessage
		));
	}
	
	/**
	* Show the user account profile page
	*
	* @access private
	* @global object $objDatabase
	*/
	function _showProfilePage()
	{
		global $objDatabase;
		
		$objResult = $objDatabase->SelectLimit("SELECT email, firstname, lastname, residence, profession, interests, webpage FROM ".DBPREFIX."access_users WHERE username='".contrexx_addslashes($_SESSION['auth']['username'])."'");
		if ($objResult !== false) {
			$this->_objTpl->setVariable(array(
				'COMMUNITY_FIRSTNAME'	=> $objResult->fields['firstname'],
				'COMMUNITY_LASTNAME'	=> $objResult->fields['lastname'],
				'COMMUNITY_RESIDENCE'	=> $objResult->fields['residence'],
				'COMMUNITY_PROFESSION'	=> $objResult->fields['profession'],
				'COMMUNITY_INTERESTS'	=> $objResult->fields['interests'],
				'COMMUNITY_WEBPAGE'	=> $objResult->fields['webpage'],
				'COMMUNITY_EMAIL'		=> $objResult->fields['email']
			));
		}
	}
}
?>
