<?PHP
/**
* Guestbook Modul
*
* frontend guestbook class
*
* @copyright CONTREXX CMS - Astalavista IT Engineering GmbH Thun
* @author Astalavista Development Team <thun@astalvista.ch>  
* @module guestbook
* @modulegroup modules
* @access public
* @version 1.0.0
*/


require_once ASCMS_MODULE_PATH . '/guestbook/Lib.class.php';

class Guestbook extends GuestbookLibrary
{	
    var $langId;
    var $_objTpl;
    var $statusMessage;
    var $arrSettings = array();
    
    		
	/**
    * Constructor
    *
    * @param  string  
    * @access public
    */   
    function Guestbook($pageContent)
    { 
    	$this->__construct($pageContent);    
    }
	

    
    /**
     * PHP5 constructor
     * @param  string  $pageContent
     * @global string  $_LANGID      
     * @access public     
     */
    function __construct($pageContent)
    {  
	    global $_LANGID;
	    $this->pageContent = $pageContent;
	    $this->langId = $_LANGID;
	    
	    $this->_objTpl = &new HTML_Template_Sigma('.');
		$this->_objTpl->setErrorHandling(PEAR_ERROR_DIE); 
		
		// get the guestbook settings
    	$this->getSettings();  
	}

	
	
	
	function getPage()
	{
    	if(!isset($_REQUEST['cmd'])){
    		$_REQUEST['cmd'] = '';
    	}
    	
    	switch($_REQUEST['cmd']){
    		case 'post':
		    	$this->_showForm();
		    	break;
		    default:
		        $this->_store();
		        $this->_showList();
		        break;
    	}
    	return $this->_objTpl->get(); 
    }   
    
    
	/**
	* shows the submit form
	*
	* @access private
	*/
	
	function _showForm()
	{	
		$this->_objTpl->setTemplate($this->pageContent, true, true);
		$this->_objTpl->setVariable("GUESTBOOK_JAVASCRIPT", $this->_getJavascript());
	}
    
    
	/**
	* Gets the guestbook status
	*
	* @global  array    $_CONFIG
	* @global  array    $_ARRAYLANG
	* @global  object   $objDatabase
	* @access private	
	*/
	
	function _showList()
	{
		global $objDatabase, $_CONFIG, $_ARRAYLANG;
		
		$this->_objTpl->setTemplate($this->pageContent, true, true);
		
		// initialize variables
	    $i = 1;
	    $paging = "";
		$pos = (isset($_GET['pos'])) ? intval($_GET['pos']) : 0;		
	
		/** start paging **/
		$query = "SELECT * 
		              FROM ".DBPREFIX."module_guestbook 
		             WHERE lang_id=".$this->langId."
		             AND status = 1
		          ORDER BY id DESC";
		$objResult = $objDatabase->Execute($query);
		$count = $objResult->RecordCount();
		$paging = getPaging($count, $pos, "&amp;section=guestbook", "<b>".$_ARRAYLANG['TXT_GUESTBOOK_ENTRIES']."</b>", false);
		/** end paging **/
		
		$this->_objTpl->setVariable("GUESTBOOK_PAGING", $paging);
        $this->_objTpl->setVariable("GUESTBOOK_TOTAL_ENTRIES", $count);
					
		$query = "SELECT * 
		              FROM ".DBPREFIX."module_guestbook
				     WHERE lang_id=".$this->langId."
				 	 AND status = 1
		          ORDER BY id DESC";
		$objResult = $objDatabase->SelectLimit($query, $_CONFIG['corePagingLimit'], $pos);
		
		while (!$objResult->EOF) {
			$class = ($i % 2) ? "row1" : "row2";
			$gender = ($objResult->fields["gender"]=="M") ? $_ARRAYLANG['guestbookGenderMale'] : $_ARRAYLANG['guestbookGenderFemale']; // N/A
			
			if ($objResult->fields['url']!=""){
				$this->_objTpl->setVariable("GUESTBOOK_URL", "<a href='".$objResult->fields['url']."'><img alt='".$objResult->fields['url']."' src='".ASCMS_MODULE_IMAGE_WEB_PATH."/guestbook/www.gif' align='baseline' border='0' /></a>");
			}
			if ($objResult->fields['email']!=""){
				$this->_objTpl->setVariable("GUESTBOOK_EMAIL", "<a href='mailto:".$objResult->fields['email']."'><img alt='".$objResult->fields["email"]."' src='".ASCMS_MODULE_IMAGE_WEB_PATH."/guestbook/email.gif' align='baseline' border='0' /></a>");
			}
			
			$this->_objTpl->setVariable(array(
					   'GUESTBOOK_ROWCLASS'   => $class,
					   'GUESTBOOK_NICK'	      => stripslashes($objResult->fields["nickname"]),
					   'GUESTBOOK_GENDER'	  => $gender,
					   'GUESTBOOK_LOCATION'	  => stripslashes($objResult->fields["location"]),
					   'GUESTBOOK_DATE'		  => $objResult->fields["datetime"],
					   'GUESTBOOK_COMMENT'	  => nl2br(stripslashes($objResult->fields["comment"])),
					   'GUESTBOOK_ID'		  => $objResult->fields["id"],
					   'GUESTBOOK_IP'		  => $objResult->fields["ip"]
			));
			$this->_objTpl->parse('guestbook_row');
			$i++;
			$objResult->MoveNext();
		}
		$this->_objTpl->setVariable("GUESTBOOK_STATUS", $this->statusMessage);
	}	

	
	
	
	/**
	* Save an entry
	*
	* @global  array    $_POST
	* @global  array    $_ARRAYLANG
	* @global  object   $objDatabase
	* @return  string   $status
	*/
	function _store()
	{
		global $objDatabase, $_ARRAYLANG;

		if(isset($_POST['nickname']) && isset($_POST['comment']) &&
		  !empty($_POST['nickname']) && !empty($_POST['comment']))
		{
			$error = "";
			$nick = htmlspecialchars(strip_tags($_POST['nickname']));
			$gender = htmlspecialchars(strip_tags($_POST['malefemale']));
			$mail = (isset($_POST['email'])&& strlen($_POST['email'])>7) ?  htmlspecialchars(strip_tags($_POST['email'])) : ""; 
			$url = (isset($_POST['url'])&& strlen($_POST['url'])>7) ? htmlspecialchars(strip_tags($_POST['url'])) : ""; 
			$comment = $this->addHyperlinking(htmlspecialchars(strip_tags($_POST['comment'])));
			$location = htmlspecialchars(strip_tags($_POST['location']));
			
			if(!empty($url)) { 
				if (!$this->isUrl($url)) $error.= $_ARRAYLANG['TXT_INVALID_INTERNET_ADDRESS']."<br />"; 
			}
			if(!empty($mail)){
				if(!$this->isEmail($mail)) $error.= $_ARRAYLANG['TXT_INVALID_EMAIL_ADDRESS']."<br />";
			}
			$status = $this->arrSettings['guestbook_activate_submitted_entries'];

			if(empty($error)){
			    $query = "INSERT INTO ".DBPREFIX."module_guestbook 
			                            (id,
			                             status,
			                             nickname,
									     gender,
									     url,
									     datetime,
									     email,
									     comment,
									     ip,
			                             location,
			                             lang_id)
					             VALUES ('',
					                     $status,
										'$nick', 
										'$gender', 
										'$url', 
										NOW(), 
										'$mail', 
										'$comment', 
										'{$_SERVER['REMOTE_ADDR']}', 
										'$location', 			    
										'$this->langId')";
			    $objDatabase->Execute($query);
			    
			    if($this->arrSettings['guestbook_send_notification_email']==1) {
			    	$this->sendNotificationEmail($nick, $comment);
			    }			    		    
			    $this->statusMessage = $_ARRAYLANG['TXT_DATA_RECORD_STORED_SUCCESSFUL']."<br />";
			} else {
			    $this->statusMessage = $error;	
			}
		}
	}

	

    /**
    * @return void
    * @desc Sends a notification email to the administrator
    */	
	function sendNotificationEmail($nick,$comment) 
	{
		global $_ARRAYLANG, $_CONFIG;
	    $message = $_ARRAYLANG['TXT_CHECK_GUESTBOOK_ENTRY']."\n\n";
	    $message .= $_ARRAYLANG['TXT_ENTRY_READS']."\n".$nick."\n".$comment;
	    $mailto = $_CONFIG['coreAdminEmail'];
	    $subject = $_ARRAYLANG['TXT_NEW_GUESTBOOK_ENTRY']." ".$_SERVER['HTTP_HOST'];
		$return = @mail($mailto, 
		                $subject, 
		                $message,
		                "From: ".$mailto."\r\n" .
		                "Reply-To: ".$mailto."\r\n" .
		                "X-Mailer: PHP/" . phpversion());
		return $return;				
	}	
}
?>